comment|*******************************************************************

Name:            Cpuid.asm
Date:            Sep,27,1994
Author:          Michael J. Steiner CIS:[76372,370]

Notes:           Contains cpuid,fpuid,cpuspd

**************************************************************************|

include          asmmacro.inc
		 .model   large

CPUIDX           macro                      ; get the CPUID information
		 db      0Fh, 0A2h          ;  into eax, ebx, ecx, edx
		 endm

C86              equ      0
C186             equ      1
C286             equ      2
C386             equ      3
C486             equ      4
F0               equ      0   ;no npu present
F87              equ      1
F287             equ      2
F387             equ      3

		 .data
control          dw       0

		 .code
		 public  _cpuid
_cpuid           proc
		 pushr   <cx,dx,ds,es>
		 mov     cl, 20h            ; load high CL value
		 mov     ax, 1              ; load a non-zero value in AX
		 shr     ax, cl             ; do the shift
		 cmp     ax, 0              ; if zero, then 86
		 jne     id186              ; jump if not 86
		 xor     al, al             ; clear al and carry flag
		 push    cs
		 db      0Fh, 14h, 0C3h
		 cmp     al, 0C3h           ; if al is C3h then 86
		 jne     idV20
		 mov     ax, C86            ; set 86 flag
		 jmp     id_exit
idV20:           pop     ax                 ; correct for lack of pop cs
		 mov     ax, 200h           ; set V20/V30 flag
		 jmp     id_exit
id186:           mov     bx, sp             ; save the current stack ptr
		 push    sp                 ; do test
		 pop     ax                 ; get the pushed value
		 cmp     ax, bx             ; did SP change ?
		 je      id286              ; if not, it's a 286+
		 mov     ax,C186            ; set 186 flag
		 jmp     id_exit
id286:           cli                        ; disable interrupts
		 pushf                      ; save the current flags
		 pushf                      ; push flags onto stack
		 pop     ax                 ; now pop flags from stack
		 or      ax, 0F000h         ; try and set bits 12-15 hi
		 push    ax
		 popf                       ; set new flags
		 pushf
		 pop     ax                 ; see if upper bits are 0
		 popf                       ; restore flags to original
		 sti                        ; enable interrupts
		 test    ax, 0F000h         ; were any upper bits 1 ?
		 jnz     id386              ; if so, not a 286
		 mov     ax,C286
		 jmp     id_exit
.386                                        ; allow 386 instructions
id386:           cli                        ; disable interrupts
		 pushfd                     ; push flags to look at
		 pop     eax                ; get eflags
		 mov     ebx, eax           ; save for later
		 xor     eax, 40000h        ; toggle bit 18
		 push    eax
		 popfd                      ; load modified eflags to CPU
		 pushfd                     ; push eflags to look at
		 pop     eax                ; get current eflags
		 push    ebx                ; push original onto stack
		 popfd                      ; restore original flags
		 sti                        ; enable interrupts
		 xor     eax, ebx           ; check if bit changed
		 jnz     id486              ; changed, so 486 or later
		 mov     ax,C386            ; set 386 flag
		 jmp     id_exit
id486:           cli                        ; disable interrupts
		 pushfd                     ; push flags to look at
		 pop     eax                ; get eflags
		 mov     ebx, eax           ; save for later
		 xor     eax, 200000h       ; toggle bit 21
		 push    eax
		 popfd                      ; load modified eflags to CPU
		 pushfd                     ; push eflags to look at
		 pop     eax                ; get current eflags
		 push    ebx                ; push original onto stack
		 popfd                      ; restore original flags
		 sti                        ; enable interrupts
		 xor     eax, ebx           ; check if bit changed
		 jnz     idpentium          ; changed, it's a pentium
		 mov     ax,C486            ; set 486 flag
		 jmp     id_exit
idpentium:       push    ecx                ; CPUID changes eax to edx
		 push    edx
		 mov     eax, 1             ; get family info. function
		 CPUIDX                     ; macro for CPUID instruction
		 and     eax, 0F00h         ; get family information
		 shr     eax, 8             ; into al
		 mov     ah, 1              ; set flag that CPUID is ok
		 pop     edx
		 pop     ecx
id_exit:         popr    <es,ds,dx,cx>
		 ret
_cpuid           endp
.8086
		 public   _fpuid
_fpuid           proc
		 push     bp
		 fninit
		 mov      byte ptr control+1,0
		 fnstcw   control                ; Store control word over 0
		 mov      ah,byte ptr control+1  ; Test contents of byte written
		 cmp      ah,03h                 ; Test for 64-bit precision flags
		 je       isfpu                  ; Got one!  Now let's find which
		 mov      ax,F0
		 jmp      fpuexit                ; No NDP found
isfpu:           and      control,not 0080h      ; IEM = 0 (interrupts on)
		 fldcw    control
		 fdisi                           ; Disable ints 287/387 will ignore
		 fstcw    control
		 test     control,0080h
		 je       over87                 ; is a 287 or 387
		 mov      ax,F87
		 jmp      fpureset
over87:          finit
		 fld1
		 fldz
		 fdiv                            ; Divide 1/0 to create infinity
		 fld      st
		 fchs                            ; Push -infinity on stack
		 fcompp                          ; Compare +-infinity
		 fstsw    control
		 mov      ax,control
		 sahf
		 jne      is387                  ; 387 will compare correctly
		 mov      ax,F287
		 jmp      fpureset
is387:           mov      ax,F387                ; test is added
fpureset:        fninit                          ; in case tests have had strange
		 finit                           ; side-effects, reset
fpuexit:         pop      bp
		 ret
_fpuid           endp


comment|********************************************************************

Proceedure:      cpuspd
Date:            Dec,21,1994
Author:          Michael J. Steiner

Notes:           8086  - 144 clocks / div
		 80186 - 144 clocks / div
		 80286 - 22 clocks / div
		 80386 - 22 clocks / div
		 80486 - 24 clocks / div
		 80586 - 25 clocks / div

		 All based on 16-BIT divide operations.

		 Given: Internal timer frequency = 1193180 Hz.
		 Example:CPUTYPE 80386

		 HERTZ = 33000000
		 MHZ = 33
		 CLOCKS = 22
		 REPS = 800 ( number of div bx statements )

		 Y = ( 1 / 1193180 ) * 1E9
		 T = 1000 / MHZ
		 O = CLOCKS * T
		 X = O * REPS
		 R = X / Y
		 C = ( R / cpuspd() ) * MHZ

		 Y - Is the minimum timer counting time in nano seconds
		 T - Is equal to 1 clock @ HERTZ
		 O - Is the time in nano seconds of 1 (DIV BX) statement
		 X - Is the total time for REPS number of (DIV BX) statements
		 R - Is the ratio of (total time spent / timer resolution)
		 C - Is the estimated CPU speed in Mhz. (can be rounded up add .5)

Prototype:       extern "C" WORD cpuspd( void );

***************************************************************************|

		 public  _cpuspd
_cpuspd          proc

; setup timer 2 to time the instruction execution time.

		 ;mov     al,10110100b            ; Timer 2,lsb,msb,mode 2,binary
		 mov     al,10110000b            ; Timer 2,lsb,msb,mode 0,binary
		 out     43h,al                  ; send command
		 iodelay
		 mov     al,0FFh                 ; counter value FFFF
		 out     42h,al                  ; send lsb to counter
		 iodelay
		 out     42h,al                  ; send msb to counter
		 iodelay

; all interrupts, including NMI, are shut off to prevent any
; interrupts from affecting the timing.

		 cli                             ; disable interrupts
		 mov     al,80h
		 out     70h,al                  ; disable NMI
		 iodelay
		 in      al,61h                  ; read the current contents
		 iodelay
		 xor     dx,dx
		 mov     bx,1
		 or      al,1                    ; set gate bit on
		 out     61h,al                  ; turn on timer (timing begins)

; execute 800 slow divide instructions bx=1,dx:ax=dx:ax\bx

		 div     bx ;100
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx ;200
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx ;300
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx ;400
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx ;500
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx ;600
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx ;700
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx ;800
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx
		 div     bx

; the timer is stopped, and interrupts are re-enabled.

		 in      al,61h                  ; read the current contents (timing ends)
		 iodelay
		 and     al,0FEh                 ; set gate bit off
		 out     61h,al                  ; disable the counter (shut timer down)
		 xor     al,al
		 out     70h,al                  ; enable NMI
		 sti                             ; enable interrupts

; now the timer contents are read, and the duration is returned

		 mov     al,80h                  ; latch output command
		 out     43h,al                  ; send command
		 iodelay
		 in      al,42h                  ; get lsb of counter
		 iodelay
		 mov     dl,al
		 in      al,42h                  ; get msb of counter
		 mov     dh,al                   ; dx = counter value
		 not     dx                      ; dx=ffff-dx
		 mov     ax,dx                   ; ax=dx=count down value
		 ret
_cpuspd          endp
		 end
